--[[
	Copyright (c) 2009-2015, Hendrik "Nevcairiel" Leppkes < h.leppkes at gmail dot com >
	All rights reserved.
]]
local _, Bartender4 = ...
local L = LibStub("AceLocale-3.0"):GetLocale("Bartender4")

local ObjectiveBarMod = Bartender4:GetModule("ObjectiveBar")

-- fetch upvalues
local Bar = Bartender4.Bar.prototype

local barregistry = Bartender4.Bar.barregistry


	-- maps option keys to function names
	local optionMap = {
		alpha = "ConfigAlpha",
		scale = "ConfigScale",
		fadeout = "FadeOut",
		fadeoutalpha = "FadeOutAlpha",
		fadeoutdelay = "FadeOutDelay",
		clickthrough = "ClickThrough",
	}

	-- retrieves a valid bar object from the barregistry table
	local function getBar(id)
		local bar = barregistry[tostring(id)]
		print(bar:GetName())
		assert(bar, ("Invalid bar id in options table. (%s)"):format(id))
		return bar
	end

	-- calls a function on the bar
	local function callFunc(bar, type, option, ...)
		local func = type .. (optionMap[option] or option)
		assert(bar[func], ("Invalid get/set function %s in bar %s."):format(func, bar.id))
		return bar[func](bar, ...)
	end

	-- universal function to get a option
	local function optGetter(info)
		local bar = getBar(info[2])
		local option = info[#info]
		return callFunc(bar, "Get", option)
	end

	-- universal function to set a option
	local function optSetter(info, ...)
		local bar = getBar(info[2])
		local option = info[#info]
		return callFunc(bar, "Set", option, ...)
	end

function ObjectiveBarMod:SetupOptions()
	if not self.options then
		self.optionobject = Bar:GetOptionObject()
		local otbl_general = {
			enabled = {
				type = "toggle",
				order = 1,
				name = L["Enabled"],
				desc = L["Enable the Objective Bar"],
				get = function() return self.db.profile.enabled end,
				set = "ToggleModule",
				handler = self,
			},
		}
		self.optionobject:AddElementGroup("general", otbl_general)
		
		self.disabledoptions = {
			general = {
				type = "group",
				name = L["General Settings"],
				cmdInline = true,
				order = 1,
				args = {
					enabled = enabled,
				}
			}
		}
		self.options = {
			order = 30,
			type = "group",
			name = L["Objective Bar"],
			desc = L["Configure the Objective Bar"],
			childGroups = "tab",
		}

		Bartender4:RegisterBarOptions("ObjectiveBar", self.options)
	end
	self.options.args = self:IsEnabled() and self.optionobject.table or self.disabledoptions
end
